--- OLD/xc/programs/Xserver/hw/sun/sunInit.c	Thu Jan  1 00:00:00 1970
+++ NEW/xc/programs/Xserver/hw/sun/sunInit.c	Thu Jan  1 00:00:00 1970
@@ -81,6 +81,7 @@
 #define CG6I NULL
 #define CG8I NULL
 #define TCXI NULL
+#define BSDTCXI NULL
 #else /* }{ */
 extern Bool sunCG3Init(
 #if NeedFunctionPrototypes
@@ -157,6 +158,21 @@
 #else /* }{ */
 #define CG8I NULL
 #endif /* } */
+/*
+ * On NetBSD, FBTYPE_RESERVED3 is the same number as FBTYPE_SUNGP3, and
+ *  that's the value returned by the kernel for the tcx.  SunOS does
+ *  not have the _RESERVED values.  So we have an init proc for the
+ *  SUNGP3 if FBTYPE_RESERVED3 and __NetBSD__ are defined....
+ */
+extern Bool sunBSDTCXInit(
+#if NeedFunctionPrototypes
+    int /* screen */,
+    ScreenPtr /* pScreen */,
+    int /* argc */,
+    char** /* argv */
+#endif
+);
+#define BSDTCXI sunBSDTCXInit
 
 #endif /* } */
 
@@ -170,10 +186,13 @@
 
 Bool sunAutoRepeatHandlersInstalled;	/* FALSE each time InitOutput called */
 Bool sunSwapLkeys = FALSE;
+Bool sunSoftCursor = FALSE;
 Bool sunFlipPixels = FALSE;
 Bool sunFbInfo = FALSE;
 Bool sunCG4Frob = FALSE;
 Bool sunNoGX = FALSE;
+const char *sunKbdDev = 0;
+const char *sunMouseDev = 0;
 
 sunKbdPrivRec sunKbdPriv = {
     -1,		/* fd */
@@ -212,8 +231,12 @@
   { NULL, "SUNGIFB" },
   { NULL, "SUNPLAS" },
 #ifdef FBTYPE_SUNGP3
+#if defined(FBTYPE_RESERVED3) && defined(__NetBSD__)
+  { BSDTCXI, "TCX under NetBSD" },
+#else
   { NULL, "SUNGP3        (cg12/gs)" },
 #endif
+#endif
 #ifdef FBTYPE_SUNGT
   { NULL, "SUNGT         (gt)" },
 #endif
@@ -521,6 +544,7 @@
 {
     static int inited;
     if (!inited) {
+	const char *dev;
 #ifndef i386
 	struct rlimit rl;
 
@@ -535,12 +559,14 @@
 	    (void) setrlimit (RLIMIT_NOFILE, &rl);
 	}
 #endif
-	sunKbdPriv.fd = open ("/dev/kbd", O_RDWR, 0);
+	dev = sunKbdDev ? sunKbdDev : "/dev/kbd";
+	sunKbdPriv.fd = open (dev, O_RDWR, 0);
 	if (sunKbdPriv.fd < 0)
-	    FatalError ("Cannot open /dev/kbd, error %d\n", errno);
-	sunPtrPriv.fd = open ("/dev/mouse", O_RDWR, 0);
+	    FatalError ("Cannot open %s, error %d\n", dev, errno);
+	dev = sunMouseDev ? sunMouseDev : "/dev/mouse";
+	sunPtrPriv.fd = open (dev, O_RDWR, 0);
 	if (sunPtrPriv.fd < 0)
-	    FatalError ("Cannot open /dev/mouse, error %d\n", errno);
+	    FatalError ("Cannot open %s, error %d\n", dev, errno);
 	getKbdType ();
 	if (sunKbdPriv.type == KB_SUN4) {
 	    (void) ioctl (sunKbdPriv.fd, KIOCLAYOUT, &sunKbdPriv.layout);
